import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

# DECLARE USER, PASWORD, AND RIVER NAME
user = 'mhalik2@gmail.com'
pswd = 'ESA.Land2023'
river_name = 'Oder'

# READ AND SAVE LIST OF ALL VIRTUAL STATIONS
vs_df = pd.read_csv(f"http://hydroweb.theia-land.fr/hydroweb/authdownload?list=rivers")
river_vs = vs_df['identifier'].loc[vs_df['river'] == river_name].to_list()


# READ WATER LEVELS, CALCULATE MIN, MAX, AND MEAN VALUES
vs_heights_list = []
for vs in river_vs:
    df = pd.read_csv(f"http://hydroweb.theia-land.fr/hydroweb/authdownload?products={vs}&format=CSV&user={user}&pwd={pswd}&mode=expert")
    st_km = int(vs[-4:])
    min_h = df['Orthometric height of water surface at reference position'].min()
    max_h = df['Orthometric height of water surface at reference position'].max()
    mean_h = df['Orthometric height of water surface at reference position'].mean()
    vs_heights_list.append([st_km, min_h, max_h, mean_h])

# CREATE DATAFRAME WITH VS HEIGHTS AND SORT IT IN ASCENDING ORDER
vs_heights = pd.DataFrame(vs_heights_list, columns=['st_km', 'min', 'max', 'mean'])
vs_heights = vs_heights.sort_values(by='st_km')
vs_heights = vs_heights.set_index('st_km')

# CREATE RIVER PROFILE PLOT
fig, ax = plt.subplots()
ax.plot(vs_heights['mean'], marker='o', color='blue', label='VS mean h')
ax.scatter(vs_heights.index, vs_heights['min'], marker='o', color='green', label='VS min h')
ax.scatter(vs_heights.index, vs_heights['max'], marker='o', color='red', label='VS max h')
ax.set_xlabel('River kilometer')
ax.set_ylabel('Altitude [m a.s.l.]')
plt.title(f'River {river_name} profile based on satellite altimetry')
plt.legend()
plt.show()

# CALCULATE WATER SURFACE SLOPES
kilometers = vs_heights.index.to_list()
slopes = []
for x in kilometers[:-1]:
    next_idx = kilometers.index(x) + 1
    km_downs, h_downs = x, vs_heights['mean'].loc[x]
    km_upstr, h_upstr = kilometers[next_idx], vs_heights['mean'].iloc[next_idx]
    slope = ((h_upstr - h_downs) / (km_upstr - km_downs)) * 1000
    slopes.append([km_downs, km_upstr, slope])


# CREATE RIVER PROFILE PLOT WITH WATER SURFACE SLOPES VARIATIONS
fig, ax = plt.subplots()
ax.plot(vs_heights['mean'], marker='o', color='blue', label='VS mean h')
ax.scatter(vs_heights.index, vs_heights['min'], marker='o', color='green', label='VS min h')
ax.scatter(vs_heights.index, vs_heights['max'], marker='o', color='red', label='VS max h')
ax.set_xlabel('River kilometer')
ax.set_ylabel('Altitude [m a.s.l.]')
ax2 = ax.twinx()
for sect in slopes:
    ax2.hlines(y=sect[2], xmin=sect[0], xmax=sect[1], color='black', label='section slope')
ax2.set_ylabel('Water surface slope [cm/km]')
ax.legend()
hand, labl = ax2.get_legend_handles_labels()
plt.title(f'River {river_name} profile based on satellite altimetry')
plt.legend(np.unique(labl))
plt.show()

# SAVE RESULTS TO CSV FILE
vs_heights.to_csv('vs_heights.csv', sep=';', decimal=',')
vs_slopes = pd.DataFrame(slopes, columns=['km_downs', 'km_upstr', 'slope [cm/km]'])
vs_slopes.to_csv('vs_slopes.csv', sep=';', decimal=',')
